"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const slash_1 = __importDefault(require("slash"));
const utils_1 = require("./utils");
const serverPluginModuleResolve_1 = require("./server/serverPluginModuleResolve");
const depOptimizer_1 = require("./depOptimizer");
const chalk_1 = __importDefault(require("chalk"));
const defaultRequestToFile = (publicPath, root) => {
    if (serverPluginModuleResolve_1.moduleRE.test(publicPath)) {
        const id = publicPath.replace(serverPluginModuleResolve_1.moduleRE, '');
        const cachedModuleFilePath = serverPluginModuleResolve_1.idToFileMap.get(id);
        if (cachedModuleFilePath) {
            return cachedModuleFilePath;
        }
        const resolved = resolveNodeModuleFile(root, id);
        if (resolved) {
            serverPluginModuleResolve_1.idToFileMap.set(id, resolved);
            return resolved;
        }
    }
    return path_1.default.join(root, publicPath.slice(1));
};
const defaultFileToRequest = (filePath, root) => {
    const moduleRequest = serverPluginModuleResolve_1.fileToRequestMap.get(filePath);
    if (moduleRequest) {
        return moduleRequest;
    }
    return `/${slash_1.default(path_1.default.relative(root, filePath))}`;
};
exports.supportedExts = ['.mjs', '.js', '.ts', '.jsx', '.tsx', '.json'];
const debug = require('debug')('vite:resolve');
exports.resolveExt = (id) => {
    const cleanId = utils_1.cleanUrl(id);
    if (!path_1.default.extname(cleanId)) {
        let inferredExt = '';
        for (const ext of exports.supportedExts) {
            try {
                // foo -> foo.js
                fs_1.default.statSync(cleanId + ext);
                inferredExt = ext;
                break;
            }
            catch (e) {
                try {
                    // foo -> foo/index.js
                    fs_1.default.statSync(path_1.default.join(cleanId, '/index' + ext));
                    inferredExt = '/index' + ext;
                    break;
                }
                catch (e) { }
            }
        }
        const queryMatch = id.match(/\?.*$/);
        const query = queryMatch ? queryMatch[0] : '';
        const resolved = cleanId + inferredExt + query;
        debug(`(extension) ${id} -> ${resolved}`);
        return resolved;
    }
    return id;
};
function createResolver(root, resolvers = [], alias = {}) {
    return {
        requestToFile: (publicPath) => {
            let resolved;
            for (const r of resolvers) {
                const filepath = r.requestToFile && r.requestToFile(publicPath, root);
                if (filepath) {
                    resolved = filepath;
                    break;
                }
            }
            if (!resolved) {
                resolved = defaultRequestToFile(publicPath, root);
            }
            resolved = exports.resolveExt(resolved);
            return resolved;
        },
        fileToRequest: (filePath) => {
            for (const r of resolvers) {
                const request = r.fileToRequest && r.fileToRequest(filePath, root);
                if (request)
                    return request;
            }
            return defaultFileToRequest(filePath, root);
        },
        alias: (id) => {
            let aliased = alias[id];
            if (aliased) {
                return aliased;
            }
            for (const r of resolvers) {
                aliased = r.alias && r.alias(id);
                if (aliased) {
                    return aliased;
                }
            }
        }
    };
}
exports.createResolver = createResolver;
exports.jsSrcRE = /\.(?:(?:j|t)sx?|vue)$|\.mjs$/;
const deepImportRE = /^([^@][^/]*)\/|^(@[^/]+\/[^/]+)\//;
function resolveBareModule(root, id, importer) {
    const optimized = resolveOptimizedModule(root, id);
    if (optimized) {
        return id;
    }
    const pkgInfo = resolveNodeModule(root, id);
    if (pkgInfo) {
        return pkgInfo.entry;
    }
    // check and warn deep imports on optimized modules
    const ext = path_1.default.extname(id);
    if (!ext || exports.jsSrcRE.test(ext)) {
        const deepMatch = id.match(deepImportRE);
        if (deepMatch) {
            const depId = deepMatch[1] || deepMatch[2];
            if (resolveOptimizedModule(root, depId)) {
                console.error(chalk_1.default.yellow(`\n[vite] Avoid deep import "${id}" since "${depId}" is a ` +
                    `pre-optimized dependency.\n` +
                    `Prefer importing from the module directly.\n` +
                    `Importer: ${importer}\n`));
            }
        }
    }
    return id;
}
exports.resolveBareModule = resolveBareModule;
const viteOptimizedMap = new Map();
function resolveOptimizedModule(root, id) {
    const cached = viteOptimizedMap.get(id);
    if (cached) {
        return cached;
    }
    const cacheDir = depOptimizer_1.resolveOptimizedCacheDir(root);
    if (!cacheDir)
        return;
    const file = path_1.default.join(cacheDir, id);
    if (fs_1.default.existsSync(file)) {
        viteOptimizedMap.set(id, file);
        return file;
    }
}
exports.resolveOptimizedModule = resolveOptimizedModule;
const nodeModulesInfoMap = new Map();
const nodeModulesFileMap = new Map();
function resolveNodeModule(root, id) {
    const cached = nodeModulesInfoMap.get(id);
    if (cached) {
        return cached;
    }
    let pkgPath;
    try {
        // see if the id is a valid package name
        pkgPath = utils_1.resolveFrom(root, `${id}/package.json`);
    }
    catch (e) { }
    if (!pkgPath) {
        // if the above resolve failed, it's either the package is not installed,
        // or the package has explicit exports field preventing us from resolving
        // its package.json. Try to resovle the package.json's path by sniffing
        // the node_modules in the path.
        try {
            const entryPath = utils_1.resolveFrom(root, id);
            if (entryPath) {
                const moduleIndex = entryPath.lastIndexOf(path_1.default.join(`node_modules`, id));
                if (moduleIndex > 0) {
                    pkgPath = path_1.default.join(entryPath.slice(0, moduleIndex), 'node_modules', id, 'package.json');
                }
            }
        }
        catch (e) { }
    }
    if (pkgPath) {
        // if yes, this is a entry import. resolve entry file
        let pkg;
        try {
            pkg = JSON.parse(fs_1.default.readFileSync(pkgPath, 'utf-8'));
        }
        catch (e) {
            return;
        }
        let entryPoint;
        if (pkg.exports) {
            if (typeof pkg.exports === 'string') {
                entryPoint = pkg.exports;
            }
            else if (pkg.exports['.']) {
                if (typeof pkg.exports['.'] === 'string') {
                    entryPoint = pkg.exports['.'];
                }
                else {
                    entryPoint = pkg.exports['.'].import;
                }
            }
        }
        if (!entryPoint) {
            entryPoint = pkg.module || pkg.main || 'index.js';
        }
        debug(`(node_module entry) ${id} -> ${entryPoint}`);
        const entryFilePath = path_1.default.join(path_1.default.dirname(pkgPath), entryPoint);
        // save resolved entry file path using the deep import path as key
        // e.g. foo/dist/foo.js
        // this is the path raw imports will be rewritten to, and is what will
        // be passed to resolveNodeModuleFile().
        entryPoint = path_1.default.posix.join(id, entryPoint);
        // save the resolved file path now so we don't need to do it again in
        // resolveNodeModuleFile()
        nodeModulesFileMap.set(entryPoint, entryFilePath);
        const result = {
            entry: entryPoint,
            entryFilePath,
            pkg
        };
        nodeModulesInfoMap.set(id, result);
        return result;
    }
}
exports.resolveNodeModule = resolveNodeModule;
function resolveNodeModuleFile(root, id) {
    const cached = nodeModulesFileMap.get(id);
    if (cached) {
        return cached;
    }
    let resolved;
    if (!path_1.default.extname(id)) {
        for (const ext of exports.supportedExts) {
            try {
                resolved = utils_1.resolveFrom(root, id + ext);
            }
            catch (e) { }
            if (resolved) {
                break;
            }
        }
    }
    if (!resolved) {
        try {
            resolved = utils_1.resolveFrom(root, id);
        }
        catch (e) { }
    }
    nodeModulesFileMap.set(id, resolved);
    return resolved;
}
exports.resolveNodeModuleFile = resolveNodeModuleFile;
//# sourceMappingURL=resolver.js.map